#!/bin/bash

bin=`dirname $0`
cd $bin
plugin_name="da2-app"

function logger() {
    if [ -n "$1" ]; then
        if [ -n "$2" ]; then
            echo "$2 : $1"
        else
            echo "INFO : $1"
        fi
    fi
}

function error_handling() {
    if [ -n "$1" ]; then
        logger "$1" "ERROR"
    fi

    if [ -e ${grafana_setting_file}.back ]; then
        if [ `wc -c < ${grafana_setting_file}.back` -gt 0 ]; then
            mv ${grafana_setting_file}.back ${grafana_setting_file}
        else
            rm -f ${grafana_setting_file}.back
        fi
    fi

    if [ -e ${plugin_path}/${plugin_name} ]; then
        rm -rf ${plugin_path}/${plugin_name}
    fi

    if [ -e ${provisioning_path}/datasources/datasource.yml ]; then
        rm -rf ${provisioning_path}/datasources/datasource.yml
    fi
    if [ -e ${provisioning_path}/plugins/plugin.yml ]; then
        rm -rf ${provisioning_path}/plugins/plugin.yml
    fi

    exit 1
}

if [ "$UID" -ne 0 ];then
    logger "execute setup.sh by root" "ERROR"
    exit 1
fi

if [ ! -e ${bin}/datasource.yml ]; then
    logger "${bin}/datasource.yml is not exist" "ERROR"
    exit 1
fi

grafana_version=$(grafana-server -v | awk -F'[ .]' '{printf "%2d%02d%02d", $2,$3,$4}')

if [ -e /etc/grafana/grafana.ini ]; then
    grafana_setting_file="/etc/grafana/grafana.ini"
else 
    echo -n "Input path of Grafana setting file: "
    read grafana_setting_file
    if [ -z "${grafana_setting_file}" ]; then
        logger "setting file is not exist" "ERROR"
        exit 1
    fi
    if [ ! -e ${grafana_setting_file} ]; then
        logger "${grafana_setting_file} is not exist" "ERROR"
        exit 1
    fi
fi

plugin_path="`grep ^'plugins =' ${grafana_setting_file} | awk -F'=' '{print $2}'`"
if [ -z "${plugin_path}" ] || [ "${plugin_path:0:1}" = ";" ]; then
    # use default dir if plugin path is not set or comment out
    plugin_path="/var/lib/grafana/plugins"
fi

if [ -e ${plugin_path}/${plugin_name} ]; then
    logger "plugin is already installed." "ERROR"
    logger "delete ${plugin_path}/${plugin_name} if you want to reinstall plugin" "ERROR"
    exit 1
fi

provisioning_path="`grep ^'provisioning =' ${grafana_setting_file} | awk -F'=' '{print $2}'`"
if [ -z "${provisioning_path}" ]; then
    # if provisioning path is not set or comment out
    provisioning_path="/etc/grafana/provisioning"
fi

cp -p ${grafana_setting_file} ${grafana_setting_file}.back || error_handling "Cannot backup Grafana setting file"
sed -i -e "s/^;http_addr =.*/http_addr = 127.0.0.1/g" ${grafana_setting_file} || error_handling
if [ "${grafana_version}" -ge 80000 ]; then
    sed -i -e "s/^;allow_loading_unsigned_plugins =\(.*\)/allow_loading_unsigned_plugins = da2-app,da2-app-datasource,da2-app-trackmap,\1/g" ${grafana_setting_file} || error_handling
fi

tar xfz ${plugin_name}.tar.gz || error_handling

if [ ! -e ${plugin_path} ]; then
    mkdir ${plugin_path} || error_handling
fi
mv ${plugin_name} ${plugin_path} || error_handling "Cannot create files for plugin"
chown -R grafana:grafana ${plugin_path}/${plugin_name}

if [ ! -e ${provisioning_path}/datasources ]; then
    mkdir -p ${provisioning_path}/datasources || error_handling
fi
cp -p ${bin}/datasource.yml ${provisioning_path}/datasources/ || error_handling "Cannot copy datasource.yml to ${provisioning_path}/datasources/"

if [ "${grafana_version}" -ge 70100 ]; then
    if [ ! -e ${provisioning_path}/plugins ]; then
        mkdir -p ${provisioning_path}/plugins || error_handling
    fi
    cp -p ${bin}/plugin.yml ${provisioning_path}/plugins/ || error_handling "Cannot copy plugin.yml to ${provisioning_path}/plugins/"
fi


logger "import plugin and datasource. wait 10 seconds."
if [ -n "`which systemctl`" ]; then
    systemctl restart grafana-server
elif [ -n "`which service`" ]; then
    service grafana-server restart
else
    logger "Cannot start Grafana becuase command to start service is not exist." "ERROR"
fi

sleep 10
if [ -z "`pidof /usr/sbin/grafana-server`" ]; then
    error_handling "Cannot start Grafana. See log file /var/log/grafana/grafana.log"
fi

logger "setup is done. Grafana is started."
rm -f ${provisioning_path}/datasources/datasource.yml 
if [ "${grafana_version}" -ge 70100 ]; then
    rm -f ${provisioning_path}/plugins/plugin.yml 
fi

